"""Base settings shared by all environments"""
# Import global settings to make it easier to extend settings.
from django.conf.global_settings import *   # pylint: disable=W0614,W0401

#==============================================================================
# Generic Django project settings
#==============================================================================

DEBUG = True
TEMPLATE_DEBUG = DEBUG

SITE_ID = 1
# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
TIME_ZONE = 'UTC'
USE_TZ = True
USE_I18N = True
USE_L10N = True
LANGUAGE_CODE = 'en'
LANGUAGES = (
    ('en', 'English'),
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = '+$l@=0=6ystdflyqticq8hsa_4t#ofipjbknb%8kn5s7www=04'

INSTALLED_APPS = (
    'salmon.core',
    'salmon.apps.monitor',

    'gunicorn',
    'south',

    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'django.contrib.admindocs',
)

#==============================================================================
# Calculation of directories relative to the project module location
#==============================================================================

import os
import sys
import salmon as project_module

PROJECT_DIR = os.path.dirname(os.path.realpath(project_module.__file__))

PYTHON_BIN = os.path.dirname(sys.executable)
ve_path = os.path.dirname(os.path.dirname(os.path.dirname(PROJECT_DIR)))
# Assume that the presence of 'activate_this.py' in the python bin/
# directory means that we're running in a virtual environment.
if os.path.exists(os.path.join(PYTHON_BIN, 'activate_this.py')):
    # We're running with a virtualenv python executable.
    VAR_ROOT = os.path.join(os.path.dirname(PYTHON_BIN), 'var')
elif ve_path and os.path.exists(os.path.join(ve_path, 'bin',
        'activate_this.py')):
    # We're running in [virtualenv_root]/src/[project_name].
    VAR_ROOT = os.path.join(ve_path, 'var')
else:
    # Set the variable root to a path in the project which is
    # ignored by the repository.
    VAR_ROOT = os.path.join(PROJECT_DIR, 'var')

if not os.path.exists(VAR_ROOT):
    os.mkdir(VAR_ROOT)

#==============================================================================
# Project URLS and media settings
#==============================================================================

ROOT_URLCONF = 'salmon.urls'

LOGIN_URL = '/login/'
LOGOUT_URL = '/logout/'
LOGIN_REDIRECT_URL = '/'

STATIC_URL = '/static/'
MEDIA_URL = '/uploads/'

STATIC_ROOT = os.path.join(VAR_ROOT, 'static')
MEDIA_ROOT = os.path.join(VAR_ROOT, 'uploads')

STATICFILES_DIRS = (
    os.path.join(PROJECT_DIR, 'static'),
)

#==============================================================================
# Database
#==============================================================================

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(VAR_ROOT, 'salmon.db'),
    }
}

#==============================================================================
# Templates
#==============================================================================

TEMPLATE_DIRS = (
    os.path.join(PROJECT_DIR, 'templates'),
)

TEMPLATE_CONTEXT_PROCESSORS += (
)

#==============================================================================
# Middleware
#==============================================================================

MIDDLEWARE_CLASSES += (
)

#==============================================================================
# Auth / security
#==============================================================================

AUTHENTICATION_BACKENDS += (
)

#==============================================================================
# Miscellaneous project settings
#==============================================================================

# work-around for https://github.com/saltstack/salt/issues/4454
SALT_COMMAND = '/usr/bin/python /usr/bin/salt'

# Web Service
WEB_HOST = 'localhost'
WEB_PORT = 9000
WEB_OPTIONS = {
    'workers': 3,
}

LOGGING = {
    'version': 1,
}

#==============================================================================
# Third party app settings
#==============================================================================



"""
sentry.management.commands.start
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2012 by the Sentry Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""
from django.core.management import call_command
from django.core.management.base import NoArgsCommand

from optparse import make_option
import sys

from salmon.core.server import SalmonHTTPServer


class Command(NoArgsCommand):
    help = 'Starts the web service'

    option_list = NoArgsCommand.option_list + (
        make_option('--debug',
            action='store_true',
            dest='debug',
            default=False),
        make_option('--noupgrade',
            action='store_false',
            dest='upgrade',
            default=True),
        make_option('--workers', '-w',
            dest='workers',
            type=int,
            default=None),
    )

    def handle(self, address=None, upgrade=True, **options):

        if address:
            if ':' in address:
                host, port = address.split(':', 1)
                port = int(port)
            else:
                host = address
                port = None
        else:
            host, port = None, None

        if upgrade:
            # Ensure we perform an upgrade before starting any service
            print "Performing upgrade before service startup..."
            call_command('migrate', verbosity=0)


        server = SalmonHTTPServer(
            debug=options.get('debug'),
            host=host,
            port=port,
            workers=options.get('workers'),
        )

        # remove command line arguments to avoid optparse failures with service code
        # that calls call_command which reparses the command line, and if --noupgrade is supplied
        # a parse error is thrown
        sys.argv = sys.argv[:1]

        print "Running web service"
        server.run()



#!/usr/bin/env python
import os
import sys

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "salmon.settings.local")

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



from django.db import models
from django.utils import timezone
from django.conf import settings


class Check(models.Model):
    """A description for a Salt call"""
    target = models.CharField(max_length=255)
    function = models.CharField(max_length=255)
    name = models.CharField(max_length=255, blank=True)
    active = models.BooleanField(default=True)
    last_run = models.DateTimeField(default=timezone.now, null=True)

    def __unicode__(self):
        return '{} {}'.format(self.target, self.function)


class Minion(models.Model):
    """A single minion surfaced by a Salt call"""
    name = models.CharField(max_length=255)

    def __unicode__(self):
        return self.name


class Result(models.Model):
    """The results of a Check"""
    check = models.ForeignKey('monitor.Check')
    minion = models.ForeignKey('monitor.Minion')
    timestamp = models.DateTimeField(default=timezone.now)
    result = models.TextField()
    result_type = models.CharField(max_length=30)
    failed = models.BooleanField(default=True)

    def __unicode__(self):
        return self.timestamp.isoformat()

    @property
    def cleaned_result(self):
        return utils.TypeTranslate(self.result_type)(self.result)


from django.shortcuts import render
from . import models


def dashboard(request):
    """Shows the latest results for each minion"""
    minions = {}
    for check in models.Check.objects.filter(active=True):
        latest_results = models.Result.objects.filter(check=check,
                                                      timestamp=check.last_run)
        for result in latest_results:
            minions.setdefault(result.minion, []).append(result)

    return render(request, 'monitor/dashboard.html', {'minions': minions})


import base64
import os
from logan.runner import run_app
from salmon.settings import base as base_settings

KEY_LENGTH = 40

def generate_settings():
    """
    This command is run when ``default_path`` doesn't exist, or ``init`` is
    run and returns a string representing the default data to put into their
    settings file.
    """
    conf_file = os.path.join(os.path.dirname(base_settings.__file__),
                             'example', 'conf.py')
    conf_template = open(conf_file).read()
    random_key = base64.b64encode(os.urandom(KEY_LENGTH))
    output = conf_template.format(default_key=random_key)
    return output


def main():
    run_app(
        project='salmon',
        default_config_path='~/.salmon/conf.py',
        default_settings='salmon.settings.base',
        settings_initializer=generate_settings,
        settings_envvar='SALMON_CONF',
    )

if __name__ == '__main__':
    main()


import json
import subprocess
from django.core.management.base import NoArgsCommand
from django.utils import timezone
from django.conf import settings
import yaml

from salmon.apps.monitor import models, utils


class Command(NoArgsCommand):
    help = "Run Salt monitor checks"

    def load_salmon_checks(self):
        checks_yaml = open(settings.SALMON_CHECKS_PATH).read()
        return yaml.safe_load(checks_yaml)


    def handle_noargs(self, **options):
        config = self.load_salmon_checks()
        self.stdout.write("Running checks...")
        for target, functions in config.items():
            for func_name, func_opts in functions.items():
                cmd = utils.build_command(target, func_name)
                check, _ = models.Check.objects.get_or_create(
                    target=target, function=func_name,
                    name=func_opts.get('name', func_name))
                self.stdout.write("+ {}".format(' '.join(cmd)))
                timestamp = timezone.now()
                result = subprocess.check_output(cmd)
                check.last_run = timestamp
                check.active = True
                check.save()
                try:
                    parsed = json.loads(result)
                except ValueError:
                    self.stdout.write("  Error parsing results")
                    continue
                # parse out minion names in the event of a wildcard target
                for name, raw_value in parsed.iteritems():
                    value = utils.parse_value(raw_value, func_opts)
                    self.stdout.write("  {}: {}".format(name, value))
                    minion, _ = models.Minion.objects.get_or_create(name=name)
                    failed = utils.check_failed(value, func_opts)
                    models.Result.objects.create(timestamp=timestamp,
                                                 result=value,
                                                 result_type=func_opts['type'],
                                                 check=check,
                                                 minion=minion,
                                                 failed=failed)



from django.core.management import call_command
from django.conf import settings


# Borrowed liberally from Sentry
# https://github.com/getsentry/sentry/blob/3cc971a843efcab317f8acac0a98f1851e8f838d/src/sentry/services/http.py
class SalmonHTTPServer(object):

    def __init__(self, host=None, port=None, debug=False, workers=None):

        self.host = host or settings.WEB_HOST
        self.port = port or settings.WEB_PORT
        self.workers = workers

        options = (settings.WEB_OPTIONS or {}).copy()
        options['debug'] = debug
        options.setdefault('bind', '%s:%s' % (self.host, self.port))
        options.setdefault('daemon', False)
        options.setdefault('timeout', 30)
        options.setdefault('proc_name', 'Salmon')
        if workers:
            options['workers'] = workers

        self.options = options

    def run(self):
        call_command('run_gunicorn', **self.options)



import os
from django.conf.urls.static import static
from django.conf.urls import patterns, url, include
from django.conf import settings

from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns('',
    (r'', include('salmon.apps.monitor.urls')),
    (r'^admin/', include(admin.site.urls)),
    (r'^static/(?P<path>.*)$', 'django.views.static.serve',
        {'document_root': os.path.join(settings.PROJECT_DIR, 'static')}),
)



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name='salmon',
    version='0.1.2',
    description="A monitoring system built on top of Salt.",
    long_description=open('README.rst').read(),
    author="Lincoln Loop",
    author_email='info@lincolnloop.com',
    url='https://github.com/lincolnloop/salmon',
    license='LICENSE',
    install_requires=[
        'django==1.5.1',
        'south==0.7.6',
        'pyyaml==3.10',
        'logan==0.5.5',
        'gunicorn>=0.17.2,<0.18.0',
    ],
    entry_points={
        'console_scripts': [
            'salmon = salmon.core.runner:main',
        ],
    },
    packages=find_packages(),
    package_data={'salmon': ['static/*.*', 'templates/*.*']},
)



from django.conf import settings


def build_command(target, function, output='json'):
    cmd_list = settings.SALT_COMMAND.split()
    cmd_list.extend(['--static', '--out={}'.format(output), target])
    cmd_list.extend(function.split())
    return cmd_list

def check_failed(value_as_str, opts):
    value = TypeTranslate(opts['type']).cast(value_as_str)
    success = eval(opts['assert'].format(value=value))
    assert isinstance(success, bool)
    # this is check_failed, not check_success
    return not success

def parse_value(raw_value, opts):
    value = raw_value
    if 'key' in opts:
        key_tree = opts['key'].split('.')
        for key in key_tree:
            value = value[key]
    return value

class TypeTranslate(object):
    def __init__(self, cast_to):
        self.cast_to = cast_to

    def cast(self, value):
        return getattr(self, 'to_{}'.format(self.cast_to))(value)

    def to_boolean(self, value):
        return bool(value)

    def to_percentage(self, value):
        return self.to_float(value.rstrip('%'))

    def to_float(self, value):
        return float(value)
